//
//  Analysis.swift
//  Trampigo
//
//  Created by Naim Ashhab on 16.12.15.
//  Copyright © 2015 Naim Ashhab. All rights reserved.
//

import UIKit

protocol AnalysablePropertyType {
  var propertyColor: UIColor { get }
}

//typealias AnalysableFilter = (Analysable) -> Bool

protocol AnalysableFilterType {
  
  typealias AnalysableObject: Analysable
  
  var filter: (AnalysableObject) -> Bool { get }
}

protocol Analysable {
  
  typealias AnalysableProperty: AnalysablePropertyType
  
  var date: NSDate! { get }
  
  static func getValueForProperty(objects: [Self], property: AnalysableProperty) -> Double
}

protocol Analysis {
  
  typealias AnalysableObject: Analysable
  
  init(objects: [AnalysableObject])
  
  var objects: [AnalysableObject] { get }
  
  func analyseProperty(property: AnalysableObject.AnalysableProperty, inTimeRange timeRange: TimeRange) -> [Double]
}

extension Analysis {
  
  func analyseProperty(property: AnalysableObject.AnalysableProperty, inTimeRange timeRange: TimeRange) -> [Double] {
    
    // TODO: could cache this sorted array
    // TODO: not use filter, but date indexes
    
    var values = [Double]()
    
    // For every date
    for date in timeRange.toDateArray() {
      let dateValue = AnalysableObject.getValueForProperty(objects.filter { $0.date.dayDate() == date.dayDate() }, property: property)
      values.append(dateValue)
    }
    
    return values
  }
  
}


class TrainingAnalysis: Analysis {
  
  // MARK: Protocol required
  
  typealias AnalysableObject = Training
  
  required init(objects: [AnalysableObject]) {
    self.objects = objects.sort({ $0.date < $1.date })
  }
  
  var objects: [AnalysableObject]

  
  // MARK: Instance
  
  var basicAnalysis: BasicTrainingAnalysis {
    return TrainingAnalysis.basicAnalyseFromTrainings(objects)
  }
  
  
  // MARK: Class
  
  static func downloadBasicAnalyseForGymnast(gymnastId: String, callback: (String, BasicTrainingAnalysis!) -> Void) {
    
    Emitter.downloadGymnastTrainingsInTimeRange(gymnastId: gymnastId, inTimeRange: TimeRange(rangeDays: 30)) {
      (gymnastId, _, trainings, error) -> Void in
      
      if let error = error {
        dump(error.localizedMessage())
      } else {
        callback(gymnastId, basicAnalyseFromTrainings(trainings))
      }
      
    }
    
  }
  
  static func basicAnalyseFromTrainings(allTrainings: [Training]) -> BasicTrainingAnalysis {
    
    let trainings = allTrainings.filter { $0.finished }
    
    // Training count
    
    let trainingsCount = Double(trainings.count)
    
    
    // Avg rounds in training
    
    let roundsSum = trainings.reduce(0) { $0 + $1.rounds.count }
    let avgRoundsInTraining = Double(roundsSum) / Double(trainings.count)
    
    
    // Avg skills in round
    
    let trainingSkillsSum = trainings.reduce(0) { $0 + $1.rounds.reduce(0) { $0 + $1.skills } }
    let avgSkillsInRound = Double(trainingSkillsSum) / Double(roundsSum)
    
    
    // Avg skill diff
    
    let diffSum = trainings.reduce(0) { $0 + $1.difficulty }
    let avgSkillDiff = diffSum / Double(trainingSkillsSum)
    
    return BasicTrainingAnalysis(trainings: trainingsCount, avgRounds: avgRoundsInTraining, avgSkillsInRound: avgSkillsInRound, avgSkillDiff: avgSkillDiff)
  }
  
}


struct BasicTrainingAnalysis {
  var trainings, avgRounds, avgSkillsInRound, avgSkillDiff: Double
}