//
//  Training.swift
//  Trampigo
//
//  Created by Naim Ashhab on 06.11.15.
//  Copyright © 2015 Naim Ashhab. All rights reserved.
//

import Foundation
import SwiftyJSON
import UIKit

typealias TrainingProperties = (rounds: Double, skills: Double, difficulty: Double, difficultyPerSkill: Double, skillsInRound: Double)

final class Training {
  
  var id            : String!
  var owner         : User!
  var date          : NSDate!
  var duration      : Int = 0 // Time interval in minutes
  var rounds        = [Round]()
  var finished      : Bool = false
  
  init(owner: User, date: NSDate!) {
    self.owner = owner
    self.date = date
  }
  
  static func countTrainingProperties(rounds: [Round]) -> TrainingProperties {
    
    let roundsCount         = Double(rounds.count)
    let skills              = Double(rounds.reduce(0) { $0 + $1.skills })
    let difficulty          = rounds.reduce(0.0) { $0 + $1.difficulty }
    let difficultyPerSkill  = difficulty / Double(skills)
    let skillsInRound       = Double(skills) / Double(roundsCount)
    
    return (roundsCount, skills, difficulty, difficultyPerSkill, skillsInRound)
  }
  
  static func countTrainingProperties(training: Training) -> TrainingProperties {
    return countTrainingProperties(training.rounds)
  }
  
}


// MARK: - Adapter

extension Training {
  
  var roundsCount         : Int { return rounds.count }
  var skills              : Int { return rounds.reduce(0) { $0 + $1.skills } }
  var difficulty          : Double { return rounds.reduce(0) { $0 + $1.difficulty } }
  var difficultyPerSkill  : Double { return difficulty / Double(skills) }
  var skillsInRound       : Double { return Double(skills) / Double(roundsCount) }
  
  
  convenience init?(json: JSON) {
    
    if json == JSON.null {
      return nil
    }
    
    guard let owner = Gymnast(json: json["owner"], accessToken: nil), let date = json["date"].double else {
      return nil
    }
    
    guard let id  = json["id"].string, let duration = json["duration"].int, let finished = json["finished"].bool else {
      return nil
    }
    
    self.init(owner: owner, date: NSDate(timeIntervalSince1970: date))
    
    self.id       = id
    self.duration = duration
    self.finished = finished
    
    for round in json["rounds"].arrayValue {
      self.rounds.append(Round(json: round))
    }
  }
  
}

extension Training: Analysable {
  
  typealias AnalysablePropertyEnum = TrainingAnalysableProperty
  
  static func getValueForProperty(objects: [Training], property: TrainingAnalysableProperty) -> Double {
    
    switch property {
    case .Duration:
      return Double(objects.reduce(0) { $0 + $1.duration })
    case .TotalRounds:
      return Double(objects.reduce(0) { $0 + $1.roundsCount })
    case .TotalSkills:
      return Double(objects.reduce(0) { $0 + $1.skills })
    case .TotalDifficulty:
      return objects.reduce(0) { $0 + $1.difficulty }
    case .AvgDifficultyOnSkill:
      let totalSkills = getValueForProperty(objects, property: .TotalSkills)
      return totalSkills == 0 ? 0 : getValueForProperty(objects, property: .TotalDifficulty) / totalSkills
    case .AvgSkillsInRound:
      let totalRounds = getValueForProperty(objects, property: .TotalRounds)
      return totalRounds == 0 ? 0 : getValueForProperty(objects, property: .TotalSkills) / totalRounds
    }
  }
  
}

enum TrainingAnalysableProperty: AnalysablePropertyType {
  case Duration, TotalRounds, TotalSkills, TotalDifficulty, AvgDifficultyOnSkill, AvgSkillsInRound
  
  var propertyColor: UIColor {
    
    switch self {
    case .Duration:
      return UIColor.redColor()
    case .TotalRounds:
      return UIColor.trampigoRoundsColor()
    case .TotalSkills:
      return UIColor.trampigoSkillsColor()
    case .TotalDifficulty:
      return UIColor.trampigoDifficultyColor()
    case .AvgDifficultyOnSkill:
      return UIColor.trampigoDifficultyPerSkillColor()
    case .AvgSkillsInRound:
      return UIColor.trampigoSkillsColor()
    }
    
  }
}

final class Round {
  
  var id: String!
  var skills: Int
  var difficulty: Double
  var difficultyPerSkill: Double {
    return difficulty / Double(skills)
  }
  
  static var propertyList: [String] {
    return ["skills", "difficulty", "difficultyPerSkill"]
  }
  
  init(skills: Int, difficulty: Double) {
    self.skills = skills
    self.difficulty = difficulty
  }
  
  init(json: JSON) {
    self.id = json["id"].stringValue
    self.skills = json["skills"].intValue
    self.difficulty = json["difficulty"].doubleValue
  }
  
  func doubleForPropertyName(property: String) -> Double? {
    
    switch property {
      
    case "skills":
      return Double(skills)
    case "difficulty":
      return difficulty
    case "difficultyPerSkill":
      return difficultyPerSkill
      
    default:
      return nil
    }
    
  }
  
  static func colorForPropertyName(property: String) -> UIColor? {
    switch property {
      
    case "skills":
      return UIColor.trampigoSkillsColor()
    case "difficulty":
      return UIColor.trampigoDifficultyColor()
    case "difficultyPerSkill":
      return UIColor.trampigoDifficultyPerSkillColor()
      
    default:
      return nil
    }
  }
  
}

struct RoundlessData {
  
  var rounds: Int
  var skills: Int
  var difficulty: Double
  
  var difficultyPerSkill: Double {
    return difficulty / Double(skills)
  }
  
  var skillsInRound: Double {
    return Double(skills) / Double(rounds)
  }
  
  init(rounds: Int, skills: Int, difficulty: Double) {
    self.rounds = rounds
    self.skills = skills
    self.difficulty = difficulty
  }
  
  init?(json: JSON) {
    
    if json == JSON.null {
      return nil
    }
    
    self.rounds = json["rounds"].intValue
    self.skills = json["skills"].intValue
    self.difficulty = json["difficulty"].doubleValue
    
  }
  
}