//
//  Coach&Gymnast.swift
//  Trampigo
//
//  Created by Naim Ashhab on 12.12.15.
//  Copyright © 2015 Naim Ashhab. All rights reserved.
//

import Foundation
import SwiftyJSON


// MARK: - Coach

final class Coach: User {
  
  // MARK: Properties
  
  var gymnasts: [Gymnast] = []
  
  
  // MARK: Inits
  
  required init?(json: JSON, accessToken: String?) {
    super.init(json: json, accessToken: accessToken)
    
    for gymnastJSON in json["connections"]["gymnasts"].arrayValue {
      if let gymnast = Gymnast(json: gymnastJSON, accessToken: nil) {
        gymnasts.append(gymnast)
      }
    }
    
  }
  
  convenience init?(json: JSON) {
    self.init(json: json["user"], accessToken: json["accessToken"].string)
  }
  
  required init(accessToken: String) {
    super.init(accessToken: accessToken)
  }
  
  
  // MARK: User defaults
  
  override func userToUserDefaultsDictionary() -> [String: AnyObject] {
    var dic = super.userToUserDefaultsDictionary()
    
    dic["gymnasts"] = gymnasts.map { (gymnast) -> [String: AnyObject] in
      gymnast.userToUserDefaultsDictionary()
    }

    return dic
  }
  
  override func fillUserDataFromDic(userDic: [String: AnyObject]) {
    super.fillUserDataFromDic(userDic)
    
    if let gymnastsDic = userDic["gymnasts"] as? [[String: AnyObject]] {
      for gymnastDic in gymnastsDic {
        if let gymnast = Gymnast(json: JSON(gymnastDic), accessToken: nil) {
          
          if let quickAnalyseDic = gymnastDic["quickStat"] as? [String: Double],
            let trainings = quickAnalyseDic["trainings"],
            let avgRounds = quickAnalyseDic["avgRounds"],
            let avgSkillsInRound = quickAnalyseDic["avgSkillsInRound"],
            let avgSkillDiff = quickAnalyseDic["avgSkillDiff"] {
              
              gymnast.quickAnalyse = BasicTrainingAnalysis(trainings: trainings, avgRounds: avgRounds, avgSkillsInRound: avgSkillsInRound, avgSkillDiff: avgSkillDiff)
          }
          
          self.gymnasts.append(gymnast)
        }
      }
    }
  }
  
  
  // MARK: Equatable
  
  override func isEqualTo(user: User?) -> Bool {
    if !super.isEqualTo(user) {
      return false
    }
    
    if let user = user as? Coach {
      
      if gymnasts.count != user.gymnasts.count {
        return false
      }
      
      for (index, gymnast) in gymnasts.enumerate() {
        if !gymnast.isEqualTo(user.gymnasts[index]) {
          return false
        }
      }
      
      return true
      
    } else {
      return false
    }
    
  }
  
}


// MARK: - Gymnast

final class Gymnast: User {
  
  // MARK: Properties
  
  var coach: Coach?
  var quickAnalyse: BasicTrainingAnalysis?
  
  
  // MARK: Inits
  
  required init?(json: JSON, accessToken: String?) {
    super.init(json: json, accessToken: accessToken)
    
    coach = Coach(json: json["connections"]["coach"], accessToken: nil)
  }
  
  convenience init?(json: JSON) {
    self.init(json: json["user"], accessToken: json["accessToken"].string)
  }
  
  required init(accessToken: String) {
    super.init(accessToken: accessToken)
  }
  
  // MARK: User defaults
  
  override func userToUserDefaultsDictionary() -> [String: AnyObject] {
    var dic = super.userToUserDefaultsDictionary()
    
    if let coachDic = self.coach?.userToUserDefaultsDictionary() {
      dic["coach"] = coachDic
    }
    
    if let quickAnalyse = quickAnalyse {
      dic["quickStat"] = ["trainings": quickAnalyse.trainings, "avgRounds": quickAnalyse.avgRounds, "avgSkillsInRound": quickAnalyse.avgSkillsInRound, "avgSkillDiff": quickAnalyse.avgSkillDiff]
    }
    
    return dic
  }
  
  override func fillUserDataFromDic(userDic: [String: AnyObject]) {
    super.fillUserDataFromDic(userDic)
    
    if let coachDic = userDic["coach"] {
      self.coach = Coach(json: JSON(coachDic), accessToken: nil)
    }
    
    if let quickStatDic = userDic["quickStat"] as? [String: Double],
      let trainings = quickStatDic["trainings"],
      let avgRounds = quickStatDic["avgRounds"],
      let avgSkillsInRound = quickStatDic["avgSkillsInRound"],
      let avgSkillDiff = quickStatDic["avgSkillDiff"] {
      
      self.quickAnalyse = BasicTrainingAnalysis(trainings: trainings, avgRounds: avgRounds, avgSkillsInRound: avgSkillsInRound, avgSkillDiff: avgSkillDiff)
    }
    
  }
  
  
  // MARK: Equatable
  
  override func isEqualTo(user: User?) -> Bool {
    if !super.isEqualTo(user) {
      return false
    }
    
    if let user = user as? Gymnast {
      
      if let coach = coach {
        return coach.isEqualTo(user.coach)
      } else {
        return user.coach == nil
      }
      
    } else {
      return false
    }
    
  }
  
}
