//
//  User.swift
//  Trampigo
//
//  Created by Naim Ashhab on 03.11.15.
//  Copyright © 2015 Naim Ashhab. All rights reserved.
//

import UIKit
import SwiftyJSON
import KeychainAccess

protocol UserEquatable {
  func isEqualTo(user: User?) -> Bool
}

class User: UserEquatable {
  
  // MARK: Properties
  
  var id          : String = ""
  var fullName    : String = ""
  var email       : String = ""
  var accessToken : String?
  
  
  // MARK: Inits
  
  required init?(json: JSON, accessToken: String?) {
    
    if json == JSON.null {
      return nil
    }
    
    guard let id = json["id"].string, let email = json["email"].string, let name = json["fullName"].string else {
      return nil
    }
    
    self.id           = id
    self.email        = email
    self.fullName     = name
    self.accessToken  = accessToken
  }
  
  required init(accessToken: String) {
    self.accessToken = accessToken
    self.loadUserFromDefaults()
    self.loadUserDataFromServer()
  }
  
  
  // MARK: User defaults
  
  private static var TRAMPIGO_USER_DEFAULTS_LOGED_USER = isGymnastTarget ? "TRAMPIGO_USER_DEFAULTS_LOGED_USER" : "TRAMPIGO_COACH_USER_DEFAULTS_LOGED_USER"
  private var TRAMPIGO_USER_DEFAULTS_LOGED_USER: String {
    return User.TRAMPIGO_USER_DEFAULTS_LOGED_USER
  }
  
  /// Save user data to user defaults
  class func saveUserToDefaults(user: User?) {
    NSUserDefaults.standardUserDefaults().setObject(user?.userToUserDefaultsDictionary(), forKey: TRAMPIGO_USER_DEFAULTS_LOGED_USER)
  }
  
  /// Load user data from user defaults
  func loadUserFromDefaults() {
    if let userDic = NSUserDefaults.standardUserDefaults().objectForKey(TRAMPIGO_USER_DEFAULTS_LOGED_USER) as? [String: AnyObject] {
      
      fillUserDataFromDic(userDic)
    }
  }
  
  func fillUserDataFromDic(userDic: [String: AnyObject]) {
    self.id       = userDic["id"] as? String ?? ""
    self.email    = userDic["email"] as? String ?? ""
    self.fullName = userDic["fullName"] as? String ?? ""
  }
  
  func userToUserDefaultsDictionary() -> [String: AnyObject] {
    return ["id": self.id, "email": self.email, "fullName": self.fullName]
  }
  
  
  // MARK: Equatable
  
  func isEqualTo(user: User?) -> Bool {
    
    if id != user?.id {
      return false
    }
    
    if email != user?.email {
      return false
    }
    
    if fullName != user?.fullName {
      return false
    }
    
    return true
  }

}


// MARK: - Sync user by server

extension User {
  
  private func loadUserDataFromServer() {
    
    self.me { (user, error) -> Void in
      
      if let error = error {
        dump(error.localizedMessage())
      }
      else if let user = user {
        
        if !user.isEqualTo(self) {
          
          User.logedUser = user
          User.saveUserToDefaults(User.logedUser)
          NSNotificationCenter.defaultCenter().postNotificationName(UserNotification.DataChanged.rawValue, object: user)
          print("User data changed. Notification posted...")
        }
      }
      
    }
  }
  
}


// MARK: - Shared instance

extension User {
  
  struct SharedInstance {
    static var instance: User!
  }
  
  static var logedUser: User! {
    get {
      return SharedInstance.instance
    }
    set {
      
      // When autologin from keychain and defaults and data from server changed, access token passed in new object and should be same
      if SharedInstance.instance?.id == newValue?.id && newValue?.accessToken == nil {
        newValue?.accessToken = SharedInstance.instance?.accessToken
      } else {
        Keychain(service: KEYCHAIN_SERVICE_TRAMPIGO)["x-access-token"] = newValue?.accessToken
      }
      
      SharedInstance.instance = newValue
      
      if let token = newValue?.accessToken {
        Emitter.sharedInstance.connectSocketWithAccessToken(token)
      } else {
        Emitter.socket = nil
      }
      
      User.saveUserToDefaults(newValue)
    }
  }
  
  static var isLoged: Bool {
    return logedUser != nil
  }
  
}


// MARK: - Login & SignUp

extension User {
  
  static func loginUser(email: String, password: String, onReturn: (user: User!, error: GeneralErrorType?) -> Void) {
    let request = UserHTTPRequest.Login(email: email, password: password)
    Parser.makeRequest(request, onReturn: { (status, data, error) -> Void in
      
      if let error = error {
        onReturn(user: nil, error: error)
      }
      else if let json = data {
        
        let user = self.init(json: json["user"], accessToken: json["accessToken"].string)
        User.logedUser = user
        onReturn(user: user, error: nil)
      }
      
    })
  }
  
  class func signUpUser(email: String, fullName: String, password: String, onReturn: (user: User!, error: GeneralErrorType?) -> Void) {
    let request = UserHTTPRequest.SignUp(email: email, password: password, fullName: fullName)
    Parser.makeRequest(request, onReturn: { (status, data, error) -> Void in
      
      if let error = error {
        onReturn(user: nil, error: error)
      }
      else if let json = data {
        
        let user = self.init(json: json["user"], accessToken: json["accessToken"].string)
        User.logedUser = user
        onReturn(user: user, error: nil)
      }
      
    })
  }
  
  class func logout() {
    logedUser = nil
    // TODO: clear all saved core data
  }
  
}


// MARK: - Me API

extension User {
  
  func me(onReturn: (user: User!, error: GeneralErrorType?) -> Void) {
    
    let request = UserHTTPRequest.Me(accessToken: self.accessToken ?? "")
    Parser.makeRequest(request) { (status, data, error) -> Void in
      
      if let error = error {
        onReturn(user: nil, error: error)
      }
      else if let json = data {
        
        let user = self.dynamicType.init(json: json["user"], accessToken: json["accessToken"].string)
        onReturn(user: user, error: nil)
      }
      
    }
    
  }
  
}

