//
//  Emitter.swift
//  Trampigo
//
//  Created by Naim Ashhab on 13.01.16.
//  Copyright © 2016 Naim Ashhab. All rights reserved.
//

import Foundation
import SwiftyJSON
import SocketIOClientSwift

// MARK: - Emit protocol

let defaultEmitTimeout: UInt64 = 5
typealias EmitStruct = (emit: APIEmit, timeout: UInt64, callback: GeneralEmitCallback?)
typealias GeneralEmitCallback = (JSON!, GeneralErrorType?) -> Void
typealias GeneralSocketEventCallback = (JSON) -> Void

protocol APIEmit {
  
  func eventName() -> String
  func items() -> AnyObject
  
}

extension APIEmit {
  
  func fire() {
    Emitter.sharedInstance.emit(self)
  }
  func fireWithAck(callback: GeneralEmitCallback) {
    Emitter.sharedInstance.emitWithAck(self, callback: callback)
  }
  
}


// MARK: - WebSocket

final class Emitter {
  
  var socket: SocketIOClient!
  
  static let sharedInstance = Emitter()
  
  class var socket: SocketIOClient! {
    get {
      return Emitter.sharedInstance.socket
    }
    set {
      Emitter.sharedInstance.socket = newValue
    }
  }
  
  class var socketStatus: SocketIOClientStatus! {
    return Emitter.socket?.status
  }
  
  class var isSocketConnected: Bool {
    if let status = Emitter.socketStatus {
      return status == .Connected
    } else {
      return false
    }
  }
  
  func connectSocketWithAccessToken(token: String) {
    
    if self.socket == nil {
      self.socket = createSocketWithAccessToken(token)
    } else {
      self.socket.disconnect()
      self.socket.options.insert(.ConnectParams(["x-access-token": token]))
    }
    
    self.socket.connect()
  }
  
  private func createSocketWithAccessToken(token: String) -> SocketIOClient {
    
    let socketURL = NSURL(string: serverURL)!
    let socket = SocketIOClient(socketURL: socketURL, options: [
      .ReconnectWait(2),
      .Reconnects(true),
      .ReconnectAttempts(3),
      .Secure(false),
      .ConnectParams(["x-access-token": token])])
    
    self.addDefaultEvents(socket)
    
    return socket
  }
  
  private func addDefaultEvents(socket: SocketIOClient) {
    
    socket.on("connect", callback: { (data, _) -> Void in
      print("Socket connected")
      
      NSNotificationCenter.defaultCenter().postNotificationName(SocketNotification.ConnectionChanged.rawValue, object: nil)
    })
    socket.on("reconnect", callback: { (data, _) -> Void in
      print("Socket reconnecting: \(data[0])")
      
      NSNotificationCenter.defaultCenter().postNotificationName(SocketNotification.ConnectionChanged.rawValue, object: nil)
    })
    socket.on("disconnect", callback: { (data, _) -> Void in
      print("Socket disconnected")
      
      NSNotificationCenter.defaultCenter().postNotificationName(SocketNotification.ConnectionChanged.rawValue, object: nil)
    })
    socket.on("error", callback: { (data, _) -> Void in
      print("Socket error: " + (data.count > 0 ? (data[0] as! String) : "Server shot down"))
    })
    
  }
  
  
  // Register emit listeners
  
  func listenSocketOn(eventName: String, callback: GeneralSocketEventCallback) {
    socket?.on(eventName) { (data, _) -> Void in
      
      print("Socket on: " + eventName)
      
      let json = JSON(data[0])
      if json != JSON.null {
        callback(json)
      }
      
    }
  }
  
  func stopListenSocketOff(eventName: String) {
    socket?.off(eventName)
  }
  
  static func listenSocketOn(eventName: String, callback: GeneralSocketEventCallback) {
    Emitter.sharedInstance.listenSocketOn(eventName, callback: callback)
  }
  static func stopListenSocketOff(eventName: String) {
    Emitter.sharedInstance.stopListenSocketOff(eventName)
  }
  
  
  // Sending emits
  
  private func emit(apiEmit: APIEmit) {
    
    if socket.status == .Connected {
      socket.emit(apiEmit.eventName(), apiEmit.items())
    } else {
      // TODO: Add to emit queue and emit on connection
    }
    
  }
  private func emitWithAck(apiEmit: APIEmit, timeout: UInt64 = defaultEmitTimeout, callback: GeneralEmitCallback) {
    
    if socket.status == .Connected {
      
      socket.emitWithAck(apiEmit.eventName(), apiEmit.items())(timeoutAfter: timeout) {
        data in
        
        if let ackJson = data[0] as? [String: AnyObject] {
          
          let json = JSON(ackJson)
          
          if json["error"] != JSON.null {
            
            let errCode = json["error"]["code"].intValue
            callback(json, ErrorProvider.errorByCode(errCode))
            
            print("Error: \(errCode): " + json["error"]["message"].stringValue)
            
          } else {
            
            callback(json, nil)
            
          }
          
        } else if let noAck = data[0] as? String where noAck == "NO ACK" {
          
          callback(JSON.null, GeneralError.NoAck)
          
        } else {
          
          dump(data)
          assert(false, "Server returned wrong format ack")
        }
        
      }
      
    } else {
      // TODO: Add to emit queue and emit on connection
    }
    
    
  }
  
}

extension Emitter {
  
  static func downloadGymnastTrainingsInTimeRange(gymnastId id: String, inTimeRange timeRange: TimeRange, callback: (String, TimeRange, [Training], GeneralErrorType?) -> Void) {
    
    
    TrainingEmit.List(id: id, from: timeRange.from, to: timeRange.to)
      .fireWithAck({
        (json, error) -> Void in
        
        if let error = error {
          callback(id, timeRange, [], error)
        } else {
          
          let trainings = json["trainings"].arrayValue.map({ (trainingJson) -> Training in
            return Training(json: trainingJson)!
          })
          
          callback(id, timeRange, trainings, nil)
          
        }
        
      })
    
  }
  
}