//
//  AnalysisVC.swift
//  Trampigo
//
//  Created by Naim Ashhab on 14.12.15.
//  Copyright © 2015 Naim Ashhab. All rights reserved.
//

import UIKit

final class AnalysisVC: UIViewController {
  
  // MARK: - Outlets
  
  @IBOutlet weak var customNavigationBar: UIView!
  @IBOutlet weak var customNavigationBarTitle: UILabel!
  
  @IBOutlet var analysisValueView: [ValueView]!
  
  @IBOutlet weak var rangeSizeSegmentControl: SegmentedControl!
  @IBOutlet weak var timeRangeLabel: UILabel!
  
  @IBOutlet weak var userSelectionCollectionView: UICollectionView!
  @IBOutlet weak var userSelectionCollectionViewHeight: NSLayoutConstraint!
  
  @IBOutlet weak var roundsTrainingPartButton   : UIButton!
  @IBOutlet weak var skillsTrainingPartButton   : UIButton!
  @IBOutlet weak var skillDiffTrainingPartButton: UIButton!
  
  @IBOutlet weak var chartView: AnalysisChartView!
  
  // MARK: - Property
  
  var timeRange = TimeRange()
  var analysis: TrainingAnalysis = TrainingAnalysis(objects: [])
  var selectedGymnastId: String!
  
  var numberOfSelectedGymnasts: Int {
    return userSelectionCollectionView.indexPathsForSelectedItems()?.count ?? 0
  }
  
  // MARK: - Lifecycle
  
  override func viewDidLoad() {
    super.viewDidLoad()
    
    self.setNeedsStatusBarAppearanceUpdate()
    
    setupView()
  }
  
  override func viewWillAppear(animated: Bool) {
    super.viewWillAppear(animated)
    
    updateView()
  }
  
  override func preferredStatusBarStyle() -> UIStatusBarStyle {
    return .LightContent
  }
  
  // MARK: - Actions
  
  @IBAction func propertyButtonPressed(sender: AnyObject) {
    
    if let button = sender as? UIButton, let property = associatedProperty(button) where chartView.showingAnyPropertyWithoutProperty(property) && numberOfSelectedGymnasts < 2 {
      
      let showing = chartView.showingProperty(property)
      chartView.setShowProperty(property, show: !showing)
      button.backgroundColor = !showing ? property.propertyColor : property.propertyColor.colorWithAlphaComponent(0.5)
      
    }
    
  }
  
  // MARK: - Selectors
  // MARK: - Private
  
  private func associatedProperty(button: UIButton) -> TrainingAnalysableProperty? {
    
    switch button {
      
    case roundsTrainingPartButton:
      return .TotalRounds
    case skillsTrainingPartButton:
      return .TotalSkills
    case skillDiffTrainingPartButton:
      return .AvgDifficultyOnSkill
      
    default:
      return nil
    }
    
  }
  
  private func updateView() {
    
    // Download training data
    
    if let selectedGymnastId = selectedGymnastId {
      downloadGymnastTrainingsInTimeRange(timeRange, gymnastId: selectedGymnastId) {
        [weak self] (_, timeRange, trainings) -> Void in
        
        self?.analysis = TrainingAnalysis(objects: trainings)
        self?.chartView.trainingAnalysis = self?.analysis
        self?.chartView.update(self?.timeRange)
        self?.fillTrainingsBasicAnalyse(self?.analysis.basicAnalysis)
      }
    }
    
    
    // Time range label
    
    timeRangeLabel.text = timeRange.from.date() + " - " + timeRange.to.date()
    
  }
  
  private func setupView() {
    
    // Cusotm navigation bar
    
    customNavigationBar.backgroundColor = UIColor.trampigoPrimaryColor()
    
    
    // Analysis value view
    
    analysisValueView[0].label = "trainings"
    analysisValueView[1].label = "avg. rounds"
    analysisValueView[2].label = "avg. skills in round"
    analysisValueView[3].label = "avg. skill diff"
    
    analysisValueView[0].color = UIColor.trampigoTrainingsColor()
    analysisValueView[1].color = UIColor.trampigoRoundsColor()
    analysisValueView[2].color = UIColor.trampigoSkillsColor()
    analysisValueView[3].color = UIColor.trampigoDifficultyPerSkillColor()
    
    analysisValueView[0].showDecimals = false
    analysisValueView[1].showDecimals = false
    analysisValueView[2].showDecimals = true
    analysisValueView[3].showDecimals = true
    
    let labelInset: CGFloat = 0
    analysisValueView[0].labelBottomInset = labelInset
    analysisValueView[1].labelBottomInset = labelInset
    analysisValueView[2].labelBottomInset = labelInset
    analysisValueView[3].labelBottomInset = labelInset
    
    
    // Segment control
    
    rangeSizeSegmentControl.unselectedLabelColor = UIColor.trampigoPrimaryColor()
    rangeSizeSegmentControl.thumbColor = UIColor.trampigoPrimaryColor()
    
    
    // User selection collectino view
    
    if isCoachTarget {
      userSelectionCollectionView.dataSource = self
      userSelectionCollectionView.delegate = self
      userSelectionCollectionView.registerNib(UINib(nibName: "UserPickCell", bundle: nil), forCellWithReuseIdentifier: UserPickCell.reuseIdentifier)
      userSelectionCollectionView.contentInset = UIEdgeInsets(top: 10, left: 20, bottom: 10, right: 20)
      userSelectionCollectionView.layoutMargins = UIEdgeInsets(top: 10, left: 50, bottom: 10, right: 50)
    } else {
      userSelectionCollectionViewHeight.constant = 0
    }
    userSelectionCollectionView.allowsMultipleSelection = true
    userSelectionCollectionView.allowsSelection = true
    userSelectionCollectionView.selectItemAtIndexPath(NSIndexPath(forRow: 0, inSection: 0), animated: false, scrollPosition: .None)
    
    
    // Chart view
    
    chartView.trainingAnalysis = analysis
    chartView.defaultChartSetup()
    chartView.setShowProperty(.TotalRounds, show: true)
    chartView.setShowProperty(.TotalSkills, show: true)
    
    
    // Training property buttons
    
    roundsTrainingPartButton.setTitle(nil, forState: .Normal)
    roundsTrainingPartButton.layer.cornerRadius = roundsTrainingPartButton.frame.height / 2
    roundsTrainingPartButton.backgroundColor = chartView.showingProperty(.TotalRounds) ? UIColor.trampigoRoundsColor() : UIColor.trampigoRoundsColor().colorWithAlphaComponent(0.5)
    
    skillsTrainingPartButton.setTitle(nil, forState: .Normal)
    skillsTrainingPartButton.layer.cornerRadius = skillsTrainingPartButton.frame.height / 2
    skillsTrainingPartButton.backgroundColor = chartView.showingProperty(.TotalSkills) ? UIColor.trampigoSkillsColor() : UIColor.trampigoSkillsColor().colorWithAlphaComponent(0.5)
    
    skillDiffTrainingPartButton.setTitle(nil, forState: .Normal)
    skillDiffTrainingPartButton.layer.cornerRadius = skillDiffTrainingPartButton.frame.height / 2
    skillDiffTrainingPartButton.backgroundColor = chartView.showingProperty(.AvgDifficultyOnSkill) ? UIColor.trampigoDifficultyPerSkillColor() : UIColor.trampigoDifficultyPerSkillColor().colorWithAlphaComponent(0.5)
    
    
    // Time range
    
    rangeSizeSegmentControl.addTarget(self, action: "timeRangeSegmentedControlValueChanged:", forControlEvents: .ValueChanged)
    
    if let coach = User.logedUser as? Coach {
      selectedGymnastId = coach.gymnasts.first?.id
    } else if let gymnast = User.logedUser as? Gymnast {
      selectedGymnastId = gymnast.id
    }
  }
  
  private func fillTrainingsBasicAnalyse(analysis: BasicTrainingAnalysis?) {
    
    analysisValueView[0].value = analysis?.trainings ?? 0
    analysisValueView[1].value = analysis?.avgRounds ?? 0
    analysisValueView[2].value = analysis?.avgSkillsInRound ?? 0
    analysisValueView[3].value = analysis?.avgSkillDiff ?? 0
    
  }
  
}


// MARK: - Networking

extension AnalysisVC {
  
  private func downloadGymnastTrainingsInTimeRange(timeRange: TimeRange, gymnastId: String, callback: (String, TimeRange, [Training]) -> Void) {
    
    Emitter.downloadGymnastTrainingsInTimeRange(gymnastId: gymnastId, inTimeRange: timeRange) {
      (gymnastId, timeRange, trainings, error) -> Void in
      
      if let error = error {
        dump(error.localizedMessage())
      } else {
        callback(gymnastId, timeRange, trainings)
      }
      
    }
    
  }
  
}


// MARK: - Time range changes

extension AnalysisVC {
  
  func timeRangeSegmentedControlValueChanged(sender: SegmentedControl!) {
    
    let rangeInDays: Int
    switch sender.selectedIndex {
      
    case 0:
      rangeInDays = 7
    case 1:
      rangeInDays = 14
    case 2:
      rangeInDays = 30
    case 3:
      rangeInDays = 90
    default:
      rangeInDays = 7
    }
    
    self.timeRange.setDayRange(rangeInDays)
    updateView()
    
  }
  
}


// MARK: - Collection view data source

extension AnalysisVC: UICollectionViewDataSource {
  
  func numberOfSectionsInCollectionView(collectionView: UICollectionView) -> Int {
    return 1
  }
  
  func collectionView(collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
    return (User.logedUser as? Coach)?.gymnasts.count ?? 0
  }
  
  func collectionView(collectionView: UICollectionView, cellForItemAtIndexPath indexPath: NSIndexPath) -> UICollectionViewCell {
    
    let cell = collectionView.dequeueReusableCellWithReuseIdentifier(UserPickCell.reuseIdentifier, forIndexPath: indexPath) as! UserPickCell
    
    cell.avatar.image = UIImage(named: "avatar-naim")
    
    return cell
    
  }
  
}


// MARK: - Collection view delegate

extension AnalysisVC: UICollectionViewDelegate {
  
  func collectionView(collectionView: UICollectionView, shouldDeselectItemAtIndexPath indexPath: NSIndexPath) -> Bool {
    return numberOfSelectedGymnasts > 1
  }
  
  func collectionView(collectionView: UICollectionView, didSelectItemAtIndexPath indexPath: NSIndexPath) {
    
    let coach = User.logedUser as! Coach
    selectedGymnastId = coach.gymnasts[indexPath.row].id
    updateView()
    
    for deselectIndexPath in collectionView.indexPathsForSelectedItems()! where indexPath != deselectIndexPath {
      collectionView.deselectItemAtIndexPath(deselectIndexPath, animated: true)
    }
    
  }
  
  
}