 //
//  MainVC.swift
//  Trampigo
//
//  Created by Naim Ashhab on 04.11.15.
//  Copyright © 2015 Naim Ashhab. All rights reserved.
//

import UIKit
import SocketIOClientSwift
import SwiftyJSON

final class MainVC: UIViewController {
  
  // MARK: - Outlets
  
  @IBOutlet var quickStatViews: [ValueView]!
  
  @IBOutlet weak var startTrainingButton  : SlantButton!
  @IBOutlet weak var trainingListButton   : SlantButton!
  @IBOutlet weak var statsButton          : SlantButton!
  
  @IBOutlet weak var gymnastsTableView: UITableView!
  
  // MARK: - Property
  
  weak var settingsBarButtonItem: UIBarButtonItem!
  weak var gymnastTableRefreshControl: UIRefreshControl!
  var runningTrainings = [String: String]() // gymnastID: trainingID
  
  // MARK: - Lifecycle
  
  override func viewDidLoad() {
    super.viewDidLoad()
    
    setupView()
    self.navigationController?.navigationBar.barStyle = UIBarStyle.Black
    
    if User.isLoged {
      setupCoachEventHandlers()
      updateTrainingGymnast()
    }
    
  }
  
  override func viewWillAppear(animated: Bool) {
    super.viewWillAppear(animated)
    
    addNotificationObserver()
    updateView()
  }
  
  override func viewDidAppear(animated: Bool) {
    super.viewDidAppear(animated)
    
    if !User.isLoged {
      performSegueWithIdentifier(MainSegue.NoLogedUser.rawValue, sender: nil)
      removeCoachEventHandlers()
    }
    
  }
  
  override func viewWillDisappear(animated: Bool) {
    NSNotificationCenter.defaultCenter().removeObserver(self)
  }
  
  override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
    
    if segue.identifier == MainSegue.Training.rawValue {
      
      if let trainingVc = segue.destinationViewController as? TrainingVC, let training = sender as? Training {
        trainingVc.training = training
      }
      
    }
    
  }
  
  
  // MARK: - Actions
  
  @IBAction func startNewTraining(sender: AnyObject) {
    
    // Start real-time training
    
    let training = Training(owner: User.logedUser!, date: NSDate())
    TrainingEmit.Start(training: training).fireWithAck {
      [weak self] (json, error) in
      
      
      if let error = error where error is TrainingError {
        
        
        // Unfinished training error
        
        if error as! TrainingError == TrainingError.UnfinishedTraining {
          
          let alert = UIAlertController(
            title: "Unfinished training",
            message: "You have some unfinished trainig. It has to be finished first",
            preferredStyle: .Alert)
          alert.addAction(UIAlertAction(title: "OK", style: .Default, handler: {(_) -> Void in
            
            if let unfinishedTraining = Training(json: json["data"]["unfinishedTraining"]) {
              self?.performSegueWithIdentifier(MainSegue.Training.rawValue, sender: unfinishedTraining)
            }
            
          }))
          self?.presentViewController(alert, animated: true, completion: nil)
          
        } else {
          assert(false, "Unresolved training error")
        }
        
        
      } else {
        training.id = json["data"]["trainingId"].stringValue
        self?.performSegueWithIdentifier(MainSegue.Training.rawValue, sender: training)
      }
      
    }
  }
  
  // MARK: - Selectors
  
  func userDataChanged(notification: NSNotification) {
    
    // Coach
    
    gymnastsTableView?.reloadData()
    
    
    // Gymnast
    
    if let settingsButton = settingsBarButtonItem?.customView as? UIButton {
      settingsButton.setTitle(User.logedUser?.fullName, forState: .Normal)
    }
  }
  
  func socketConnectionLost(notification: NSNotification) {
    setupViewForSocketState(connected: Emitter.isSocketConnected)
  }
  
  func presentSettingsViewController() {
    performSegueWithIdentifier(MainSegue.Settings.rawValue, sender: nil)
  }
  
  // MARK: - Private
  
  private func performLogout() {
    User.logout()
    performSegueWithIdentifier(MainSegue.NoLogedUserAnimated.rawValue, sender: nil)
  }
  
  private func addNotificationObserver() {
    NSNotificationCenter.defaultCenter().addObserver(self, selector: "userDataChanged:", name: UserNotification.DataChanged.rawValue, object: nil)
    
    registerForSocketConnectionNotifications(connectionChangedSelector: "socketConnectionLost:")
  }
  
  private func updateRunningTrainingCells() {
    
    gymnastsTableView.reloadData()
    
  }
  
  private func updateTrainingGymnast() {
    
    if let gymnasts = (User.logedUser as? Coach)?.gymnasts {
      let gymnastIds = gymnasts.map({ (gymnast) -> String in
        return gymnast.id
      })
      
      TrainingEmit.Running(userIds: gymnastIds).fireWithAck({
        [weak self] (json, error) -> Void in
        
        if let error = error {
          dump(error)
        } else if let json = json where json != JSON.null {
          
          for gymnastId in gymnastIds {
            
            if let trainingId = json[gymnastId].string  {
              self?.runningTrainings[gymnastId] = trainingId
            }
            
          }
          
          self?.updateRunningTrainingCells()
        }
        
        })
    }

  }
  
  // MARK: Setup view
  
  private func setupView() {
    
    // Colors
    
    trainingListButton.backgroundColor = UIColor.trampigoPrimaryColor()
    statsButton.backgroundColor = UIColor.trampigoPrimaryColor()
    
    
    // Settings button
    
    let settingsButton = UIButton(type: UIButtonType.System)
    
    settingsButton.setTitle(User.logedUser?.fullName, forState: .Normal)
    settingsButton.titleLabel?.font = UIFont.boldSystemFontOfSize(UIFont.systemFontSize() + 3)
    settingsButton.tintColor = UIColor.whiteColor()
    settingsButton.addTarget(self, action: "presentSettingsViewController", forControlEvents: .TouchUpInside)
    settingsButton.sizeToFit()
    
    navigationItem.setLeftBarButtonItem(UIBarButtonItem(customView: settingsButton), animated: false)
    settingsBarButtonItem = navigationItem.leftBarButtonItem
    
    // Setup view via target
    
    isGymnastTarget ? setupGymnastView() : setupCoachView()
    
  }
  
  private func setupCoachView() {
    for view in quickStatViews {
      view.removeFromSuperview()
    }
    
    startTrainingButton.removeFromSuperview()
    
    gymnastsTableView.dataSource = self
    gymnastsTableView.delegate = self
    gymnastsTableView.registerNib(UINib(nibName: "GymnastTableCell", bundle: nil), forCellReuseIdentifier: GymnastTableCell.reuseIdentifier)
    gymnastsTableView.tableFooterView = UIView()
    gymnastsTableView.rowHeight = 140
    
    
    // Pull to refresh
    
    let refreshControl = UIRefreshControl()
    refreshControl.attributedTitle = NSAttributedString(string: "Pull to refresh")
    refreshControl.addTarget(self, action: "updateUserQuickStats", forControlEvents: .ValueChanged)
    gymnastsTableView.addSubview(refreshControl)
    gymnastTableRefreshControl = refreshControl
  }
  
  private func setupGymnastView() {
    
    gymnastsTableView.removeFromSuperview()
    
    // Quick stat
    
    let quickStatLabels = ["trainings", "avg. rounds", "avg. skills in round", "avg. skill diff"]
    
    for (index, view) in quickStatViews.enumerate() {
      view.label = quickStatLabels[index]
      
      // Bottom border
      if index < 2 {
        
      }
      
      // Right border
      if index % 2 == 0 {
        
      }
    }
    
  }
  
  private func setupViewForSocketState(connected connected: Bool) {
    
    // Settings bar button item
    
    if let settingsBarButtonItem = navigationItem.leftBarButtonItem, let settingsBarButton = settingsBarButtonItem.customView as? UIButton {
      
      if connected {
        settingsBarButton.setTitle(User.logedUser?.fullName, forState: .Normal)
      } else {
        settingsBarButton.setTitle("Connecting...", forState: .Normal)
      }
      settingsBarButton.sizeToFit()
      
    }
    
    
    // After connection
    
    if connected {
      updateTrainingGymnast()
      updateUserQuickStats()
    } else {
      self.fillQuickAnalyseToGymnastMainView((User.logedUser as? Gymnast)?.quickAnalyse, upToDate: false)
    }
    
    
    // Buttons
    
    startTrainingButton.enabled = connected
    trainingListButton.enabled = connected
    statsButton.enabled = connected
    
    
    // Gymnasts table view
    
    gymnastsTableView?.reloadData()
    
  }
  
  private func updateView() {
    
    setupViewForSocketState(connected: Emitter.isSocketConnected)
    
  }
}


// MARK: - Gymnast table view data source

extension MainVC : UITableViewDataSource {
  
  func numberOfSectionsInTableView(tableView: UITableView) -> Int {
    return 1
  }
  
  func tableView(tableView: UITableView, titleForHeaderInSection section: Int) -> String? {
    return "Gymnasts"
  }
  
  func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
    return (User.logedUser as? Coach)?.gymnasts.count ?? 0
  }
  
  func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
    
    let cell = tableView.dequeueReusableCellWithIdentifier(GymnastTableCell.reuseIdentifier, forIndexPath: indexPath) as! GymnastTableCell
    
    if let gymnasts = (User.logedUser as? Coach)?.gymnasts {
      
      let gymnast = gymnasts[indexPath.row]
      cell.setGymnast(gymnast)
      cell.setGymnastQuickAnalyse(gymnast.quickAnalyse)
      
      cell.isRunningTraining = runningTrainings[gymnast.id] != nil && Emitter.isSocketConnected
    }
    
    cell.userInteractionEnabled = Emitter.isSocketConnected
    
    return cell
  }
  
}


// MARK: - Gymnast table view delegate

extension MainVC : UITableViewDelegate {
  
  func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
    tableView.deselectRowAtIndexPath(indexPath, animated: true)
    
    if let cell = tableView.cellForRowAtIndexPath(indexPath) as? GymnastTableCell {
      if let trainingId = runningTrainings[cell.gymnast.id] {
        
        TrainingEmit.GetOne(trainingID: trainingId).fireWithAck({
          [weak self] (json, error) -> Void in
          
          if let error = error {
            dump(error) // TODO: Error
          } else if let training = Training(json: json["training"]) {
            self?.performSegueWithIdentifier(MainSegue.Training.rawValue, sender: training)
          }
          
        })
        
      }
    }
  }
  
}


// MARK: - Emit events

extension MainVC {
  
  private func setupCoachEventHandlers() {
    
    // Start
    
    Emitter.listenSocketOn("training/start") {
      [weak self] (json) -> Void in
      
      if let trainingId = json["trainingId"].string, let gymnastId = json["gymnastId"].string {
        self?.runningTrainings[gymnastId] = trainingId
        self?.updateRunningTrainingCells()
      }
      
    }
    
    
    // Finish
    
    Emitter.listenSocketOn("training/finish") {
      [weak self] (json) -> Void in
      
      if let gymnastId = json["id"].string {
        self?.runningTrainings.removeValueForKey(gymnastId)
        self?.updateUserQuickStats()
        self?.updateRunningTrainingCells()
        self?.dismissViewController()
      }
      
    }
    
    
    // Cancel
    
    Emitter.listenSocketOn("training/cancel"){
      [weak self] (json) -> Void in
      
      if let gymnastId = json["id"].string {
        self?.runningTrainings.removeValueForKey(gymnastId)
        self?.updateRunningTrainingCells()
        self?.dismissViewController()
      }
      
    }
    
  }
  
  private func removeCoachEventHandlers() {
    
    Emitter.stopListenSocketOff("training/start")
    Emitter.stopListenSocketOff("training/cancel")
    Emitter.stopListenSocketOff("training/finish")
  }
  
}


// MARK: - Quick stat data

extension MainVC {
  
  func updateUserQuickStats() {
    
    if let logedUser = User.logedUser {
      if isGymnastTarget {
        
        TrainingAnalysis.downloadBasicAnalyseForGymnast(logedUser.id, callback: {
          [weak self] (_, quickAnalyse) -> Void in
          
          (logedUser as! Gymnast).quickAnalyse = quickAnalyse
          self?.fillQuickAnalyseToGymnastMainView(quickAnalyse)
          
        })
        
      } else {
        
        for (index, gymnast) in (logedUser as! Coach).gymnasts.enumerate() {
          
          TrainingAnalysis.downloadBasicAnalyseForGymnast(gymnast.id, callback: {
            [weak self] (gymnastId, quickAnalyse) -> Void in
            
            gymnast.quickAnalyse = quickAnalyse
            self?.gymnastTableRefreshControl.endRefreshing()
            
            
            // Update proper visible cell
            
            if let cell = self?.gymnastsTableView.cellForRowAtIndexPath(NSIndexPath(forRow: index, inSection: 0)) as? GymnastTableCell {
              
              if self!.gymnastsTableView.visibleCells.contains(cell) {
                cell.setGymnastQuickAnalyse(quickAnalyse)
              }
              
            }

            
            
          })
          
        }
        
      }
    }
    
  }
  
  private func fillQuickAnalyseToGymnastMainView(basicAnalysis: BasicTrainingAnalysis!, upToDate: Bool = true) {
    
    self.quickStatViews[0].value = basicAnalysis?.trainings ?? 0
    self.quickStatViews[1].value = basicAnalysis?.avgRounds ?? 0
    self.quickStatViews[2].value = basicAnalysis?.avgSkillsInRound ?? 0
    self.quickStatViews[3].value = basicAnalysis?.avgSkillDiff ?? 0
    
    for quickStatView in quickStatViews {
      quickStatView.upToDate = upToDate
    }
    
  }
}