//
//  TrainingVC.swift
//  Trampigo
//
//  Created by Naim Ashhab on 06.11.15.
//  Copyright © 2015 Naim Ashhab. All rights reserved.
//

import UIKit
import MGSwipeTableCell
import SocketIOClientSwift
import SwiftyJSON

final class TrainingVC: UIViewController {
  
  // MARK: - Outlets
  
  @IBOutlet weak var cancelButton : UIButton!
  @IBOutlet weak var saveButton   : UIButton!
  
  @IBOutlet weak var timeLabel  : UILabel!
  @IBOutlet weak var dateLabel  : UILabel!
  
  @IBOutlet weak var gymnastAvatar: UIImageView!
  
  @IBOutlet var valueViews: [ValueView]!
  
  @IBOutlet weak var minView        : ValueView!
  @IBOutlet weak var roundsView     : ValueView!
  @IBOutlet weak var difficultyView : ValueView!
  @IBOutlet weak var skillsView     : ValueView!
  @IBOutlet weak var diffSkillView  : ValueView!
  
  @IBOutlet weak var pageControl: UIPageControl!
  
  @IBOutlet weak var mainScrollView   : UIScrollView!
  @IBOutlet weak var roundsTableView  : UITableView!
  @IBOutlet weak var newRoundButton   : SlantButton!
  @IBOutlet weak var chartView        : TrainingChartView!
  
  @IBOutlet weak var skillsChartButton        : UIButton!
  @IBOutlet weak var diffPerSkillChartButton  : UIButton!
  
  // MARK: - Property
  
  /// Training to present
  var training: Training!
  var inProgress: Bool = true
  
  /// Loged user is training owner?
  var isTrainingOwner: Bool {
    return User.logedUser?.id == training.owner?.id
  }
  
  /// Duration training timer
  private var timer: NSTimer?
  
  // MARK: - Lifecycle
  
  override func viewDidLoad() {
    super.viewDidLoad()
    
    assert(training != nil, "Training has to be set before the view is loaded")
    
    setupView()
    setupRoundsTableView()
    setupTimer()
    
  }
  
  override func viewWillAppear(animated: Bool) {
    super.viewWillAppear(animated)
    
    if isCoachTarget {
      addTrainingEventHandlers()
    }
    
    registerForSocketConnectionNotifications(connectionChangedSelector: "socketConnectionStatusChanged:")
    
    updateView()
  }
  
  override func viewWillDisappear(animated: Bool) {
    super.viewWillDisappear(animated)
    
    if isCoachTarget {
      removeTrainingEventHandlers()
    }
    
    NSNotificationCenter.defaultCenter().removeObserver(self)
    
  }
  
  override func prefersStatusBarHidden() -> Bool {
    return true
  }
  
  override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
    if segue.identifier == TrainingSegue.NewRound.rawValue, let vc = segue.destinationViewController as? NewRoundVC {
      
      vc.roundNumber = training.rounds.count + 1
      vc.onReturn = {
        [weak self] (skills, difficulty) -> Void in
        
        let newRound = Round(skills: skills, difficulty: difficulty)
        self?.training.rounds.append(newRound)
        if let training = self?.training {
          TrainingEmit.AddRound(training: training, round: newRound).fire()
        }
        
        self?.dismissViewController()
        
      }
      
    }
    else if segue.identifier == TrainingSegue.EditRound.rawValue, let vc = segue.destinationViewController as? NewRoundVC, let cell = sender as? TrainingRoundCell, let indexPath = roundsTableView.indexPathForCell(cell) {
      
      vc.roundNumber = indexPath.row + 1
      let round = training.rounds[indexPath.row]
      vc.initialValues = (round.skills, round.difficulty)
      vc.onReturn = {
        [weak self] (skills, difficulty) -> Void in
        
        let round = self?.training.rounds[indexPath.row]
        round?.skills = skills
        round?.difficulty = difficulty
        
        if let training = self?.training, let round = round {
          let emit = TrainingEmit.EditRound(training: training, index: indexPath.row, round: round)
          dump(emit.items())
          emit.fire()
        }
        
        self?.dismissViewController()
      }
    }
    
  }
  
  // MARK: - Actions
  
  @IBAction func cancel(sender: AnyObject) {
    
    // Not owner
    
    if !isTrainingOwner || !inProgress {
      return self.dismissViewController()
    }
    
    
    // Owner 
    
    func processCancel() {
      self.timer?.invalidate()
      TrainingEmit.Canceled(training: self.training).fire()
      self.dismissViewController()
    }
    
    if self.training.rounds.count > 0 {
      
      let alert = UIAlertController(title: "Delete training".localized, message: "All training data will be deleted ...for ever...".localized, preferredStyle: .ActionSheet)
      let notCancelAction = UIAlertAction(title: "Keep training".localized, style: .Cancel, handler: nil)
      let cancelAction = UIAlertAction(title: "Delete".localized, style: .Destructive, handler: {
        (_) -> Void in
        
        processCancel()
        
      })
      
      alert.addAction(cancelAction)
      alert.addAction(notCancelAction)
      presentViewController(alert, animated: true, completion: nil)
    }
    else {
      processCancel()
    }
  }
  
  @IBAction func save(sender: AnyObject) {
    timer?.invalidate()
    TrainingEmit.Finished(training: self.training).fire()
    self.dismissViewController()
  }
  
  @IBAction func addNewRound(sender: AnyObject) {
    self.performSegueWithIdentifier(TrainingSegue.NewRound.rawValue, sender: nil)
  }
  
  @IBAction func selectedSkillsToChart(sender: AnyObject) {
    chartView.showSkills = !chartView.showSkills
    
    if !chartView.showingAnyProperty {
      chartView.showSkills = !chartView.showSkills
    } else {
      skillsChartButton.backgroundColor = chartView.showSkills ? UIColor.trampigoSkillsColor() : UIColor.trampigoSkillsColor().colorWithAlphaComponent(0.5)
      chartView.update()
    }
  }
  
  @IBAction func selectedDiffPerSkillToChart(sender: AnyObject) {
    chartView.showDifficultyPerSkill = !chartView.showDifficultyPerSkill
    
    if !chartView.showingAnyProperty {
      chartView.showDifficultyPerSkill = !chartView.showDifficultyPerSkill
      return
    }
    
    diffPerSkillChartButton.backgroundColor = chartView.showDifficultyPerSkill ? UIColor.trampigoDifficultyPerSkillColor() : UIColor.trampigoDifficultyPerSkillColor().colorWithAlphaComponent(0.5)
    chartView.update()
  }
  
  @IBAction func cellLongPress(sender: AnyObject) {
    
    roundsTableView.setEditing(true, animated: true)
//    let recognizer = sender as! UILongPressGestureRecognizer
//    let position = recognizer.locationInView(roundsTableView)
//    if let indexPath = roundsTableView.indexPathForRowAtPoint(position) {
//      
//    }
  }
  
  // MARK: - Selectors
  
  func pageControlValueChanged() {
    let xOffset = CGRectGetWidth(mainScrollView.bounds) * CGFloat(pageControl.currentPage)
    
    UIView.animateWithDuration(0.3) { () -> Void in
      self.mainScrollView.contentOffset = CGPoint(x: xOffset, y: 0)
    }
  }
  
  func updateView(reloadTable: Bool = true) {
    
    
    saveButton.enabled = training.rounds.count > 0
    
    
    // Rounds
    
    if reloadTable {
      roundsTableView.reloadData()
    }
    
    
    // Data
    
    minView.value         = training.finished ? Double(training.duration) : -training.date.timeIntervalSinceNow / 60
    roundsView.value      = Double(training.roundsCount)
    difficultyView.value  = training.difficulty
    skillsView.value      = Double(training.skills)
    diffSkillView.value   = training.difficultyPerSkill
    
    // Chart
    
    chartView.update()
  }
  
  func durationTimeIntervalFired(timer: NSTimer!) {
    minView.value = -training.date.timeIntervalSinceNow / 60
  }
  
  func socketConnectionStatusChanged(notification: NSNotification) {
    setupViewForSocketState(connected: Emitter.isSocketConnected)
  }
  
  // MARK: - Private
  
  private func setupTimer() {
    if !training.finished {
      timer = NSTimer.scheduledTimerWithTimeInterval(60, target: self, selector: "durationTimeIntervalFired:", userInfo: nil, repeats: true)
    }
  }
  
  private func setupView() {
    
    pageControl.addTarget(self, action: "pageControlValueChanged", forControlEvents: .ValueChanged)
    mainScrollView.delegate = self
    
    
    // Gymnasts avatar
    
    gymnastAvatar.layer.cornerRadius = gymnastAvatar.frame.height / 2
    gymnastAvatar.clipsToBounds = true
    gymnastAvatar.hidden = isTrainingOwner
    
    
    // Header
    
    timeLabel.text = training.date.time()
    dateLabel.text = training.date.day() + " " + training.date.date()
    
    
    // Buttons
    
    newRoundButton.backgroundColor = UIColor.trampigoPrimaryColor()
    saveButton.hidden = !isTrainingOwner || !inProgress
    
    if !isTrainingOwner || !inProgress {
      newRoundButton.removeFromSuperview()
    }
    
    
    // Chart
    
    chartView.defaultChartSetup()
    chartView.training = training
    
    
    // Chart buttons
    
    skillsChartButton.setTitle(nil, forState: .Normal)
    skillsChartButton.layer.cornerRadius = skillsChartButton.frame.height / 2
    skillsChartButton.backgroundColor = chartView.showSkills ? UIColor.trampigoSkillsColor() : UIColor.trampigoSkillsColor().colorWithAlphaComponent(0.5)
    
    diffPerSkillChartButton.setTitle(nil, forState: .Normal)
    diffPerSkillChartButton.layer.cornerRadius = diffPerSkillChartButton.frame.height / 2
    diffPerSkillChartButton.backgroundColor = chartView.showDifficultyPerSkill ? UIColor.trampigoDifficultyPerSkillColor() : UIColor.trampigoDifficultyPerSkillColor().colorWithAlphaComponent(0.5)
    
    
    // Socket state
    
    setupViewForSocketState(connected: Emitter.isSocketConnected)
  }
  
  private func setupViewForSocketState(connected connected: Bool) {
    
    if isTrainingOwner && inProgress {
      newRoundButton.enabled = connected
      cancelButton.enabled = connected
      saveButton.enabled = connected && training.rounds.count > 0
      
      roundsView.upToDate = connected
      difficultyView.upToDate = connected
      skillsView.upToDate = connected
      diffSkillView.upToDate = connected
    }
    
  }
  
  private func setupRoundsTableView() {
    
    roundsTableView.registerNib(UINib(nibName: "TrainingRoundCell", bundle: nil), forCellReuseIdentifier: TrainingRoundCell.reuseIdentifier)
    
    roundsTableView.dataSource = self
    roundsTableView.delegate = self
    roundsTableView.tableFooterView = UIView()
    
    roundsTableView.contentInset = UIEdgeInsets(top: 0, left: 0, bottom: 70, right: 0) // 70 -> new round button height
  }
  
  private func addTrainingEventHandlers() {
    
    // Add round
    
    Emitter.listenSocketOn("training/add/round") {
      [weak self] (json) -> Void in
      
      if let roundSkills = json["skills"].int, let roundDiff = json["difficulty"].double where json["id"].stringValue == self?.training.id {
        
        let round = Round(skills: roundSkills, difficulty: roundDiff)
        self?.training.rounds.append(round)
        self?.updateView()
      }
      
    }
    
    
    // Edit round
    
    Emitter.listenSocketOn("training/edit/round") {
      [weak self] (json) -> Void in
      
      if let roundIndex = json["index"].int, let roundSkills = json["skills"].int, let roundDiff = json["difficulty"].double where json["id"].stringValue == self?.training.id {
        
        let round = Round(skills: roundSkills, difficulty: roundDiff)
        self?.training.rounds.removeAtIndex(roundIndex)
        self?.training.rounds.insert(round, atIndex: roundIndex)
        self?.updateView()
      }
      
    }
    
    
    // Delete round
    
    Emitter.listenSocketOn("training/delete/round") {
      [weak self] (json) -> Void in
      
      if let roundIndex = json["index"].int where json["id"].stringValue == self?.training.id {
        
        self?.training.rounds.removeAtIndex(roundIndex)
        self?.updateView()
      }
      
    }
  }
  
  private func removeTrainingEventHandlers() {
    
    Emitter.stopListenSocketOff("training/add/round")
    Emitter.stopListenSocketOff("training/edit/round")
    Emitter.stopListenSocketOff("training/delete/round")
    
  }
  
}


// MARK: - Scroll view delegate

extension TrainingVC: UIScrollViewDelegate {
  
  func scrollViewDidEndDecelerating(scrollView: UIScrollView) {
    let page = scrollView.contentOffset.x / CGRectGetWidth(scrollView.bounds)
    pageControl.currentPage = Int(page)
  }
  
}


// MARK: - Rounds table view

extension TrainingVC: UITableViewDataSource {
  
  func numberOfSectionsInTableView(tableView: UITableView) -> Int {
    return 1
  }
  
  func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
    return training.rounds.count
  }
  
  func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
    
    let cell = tableView.dequeueReusableCellWithIdentifier(TrainingRoundCell.reuseIdentifier, forIndexPath: indexPath) as! TrainingRoundCell
    
    cell.setRound(training.rounds[indexPath.row])
    
    
    // Swipe delete action
    
    let deleteSwipeButton = MGSwipeButton(title: "Delete", backgroundColor: UIColor.redColor(), callback: { (_) -> Bool in
      
      if let indexPath = tableView.indexPathForCell(cell) where !self.training.finished {
        self.training.rounds.removeAtIndex(indexPath.row)
        self.roundsTableView.deleteRowsAtIndexPaths([indexPath], withRowAnimation: .Fade)
        
        TrainingEmit.DeleteRound(training: self.training, index: indexPath.row).fire()
        self.updateView(false)
        return true
      }
      
      return false
    })
    
    
    // Swipe edit action
    
    let editSwipeButton = MGSwipeButton(title: "Edit", backgroundColor: UIColor.trampigoPrimaryColor(), callback: { (_) -> Bool in
      
      if !self.training.finished {
        self.performSegueWithIdentifier(TrainingSegue.EditRound.rawValue, sender: cell)
        return true
      }
      
      return false
    })
    
    
    if isTrainingOwner && inProgress {
      cell.rightButtons = [deleteSwipeButton, editSwipeButton]
    }
    
    
    cell.rightSwipeSettings.transition = .Static
    cell.rightExpansion.buttonIndex = 1
    
    cell.showsReorderControl = true
    
    return cell
  }
  
}

extension TrainingVC: UITableViewDelegate {
  
  func tableView(tableView: UITableView, editingStyleForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCellEditingStyle {
    return .None
  }
  
  func tableView(tableView: UITableView, canMoveRowAtIndexPath indexPath: NSIndexPath) -> Bool {
    return true
  }
  
  func tableView(tableView: UITableView, moveRowAtIndexPath sourceIndexPath: NSIndexPath, toIndexPath destinationIndexPath: NSIndexPath) {
    
    let round = training.rounds.removeAtIndex(sourceIndexPath.row)
    training.rounds.insert(round, atIndex: destinationIndexPath.row)
    tableView.setEditing(false, animated: true)
    updateView(false)
  }
  
}
