//
//  AnalysisChartView.swift
//  Trampigo
//
//  Created by Naim Ashhab on 15.12.15.
//  Copyright © 2015 Naim Ashhab. All rights reserved.
//

import UIKit
import Charts

final class AnalysisChartView: LineChartView {
  
  // MARK: - Properties
  
  var trainingAnalysis: TrainingAnalysis!
  var timeRange = TimeRange()
  var showingAnyProperty: Bool {
    
    for property in iterateEnum(TrainingAnalysableProperty) {
      if showingProperties[property] ?? false {
        return true
      }
    }
    
    return false
  }
  var numberOfShowingProperties: Int {
    
    var numberOfShowingProperties = 0
    for property in iterateEnum(TrainingAnalysableProperty) {
      if showingProperties[property] ?? false {
        numberOfShowingProperties++
      }
    }
    
    return numberOfShowingProperties
  }
  
  private var showingProperties: [TrainingAnalysableProperty: Bool] = {
    
    var dic = [TrainingAnalysableProperty: Bool]()
    for property in iterateEnum(TrainingAnalysableProperty) {
      dic[property] = false
    }
    
    return dic
  }()
  
  
  // MARK: - Getters & Setters
  
  func showingProperty(property: TrainingAnalysableProperty) -> Bool {
    return showingProperties[property] ?? false
  }
  
  func setShowProperty(property: TrainingAnalysableProperty, show: Bool) {
    showingProperties[property] = show
    update(timeRange)
  }

  func showingAnyPropertyWithoutProperty(leavedProperty: TrainingAnalysableProperty) -> Bool {
    for property in iterateEnum(TrainingAnalysableProperty) {
      if (showingProperties[property] ?? false) && property != leavedProperty {
        return true
      }
    }
    
    return false
  }
  
  
  // MARK: - Public
  
  func update(timeRange: TimeRange?) {
    clear()
    
    if let timeRange = timeRange {
      self.timeRange = timeRange
    }
    
    if let data = createChartData(self.timeRange) {
      self.data = data
    }
  }
  
  func defaultChartSetup() {
    
    noDataText = "Not enought data for chart...".localized
    infoTextColor = UIColor.trampigoPrimaryColor()
    descriptionText = ""
    rightAxis.enabled = false
    leftAxis.enabled = false
    xAxis.enabled = true
    legend.enabled = false
    drawGridBackgroundEnabled = false
    highlightPerTapEnabled = false
    highlightPerDragEnabled = false
    drawBordersEnabled = false
    scaleYEnabled = false
    scaleXEnabled = true
    
  }
  
  
  // MARK: - Private
  
  private func createChartData(timeRange: TimeRange) -> ChartData? {
    
    if trainingAnalysis == nil {
      return nil
    }
    
    var dataSets = [LineChartDataSet]()
    
    // Days date as X values
    
    var xVals = [String]()
    for day in timeRange.toDateArray() {
      xVals.append(day.date())
    }
    
    for property in iterateEnum(TrainingAnalysableProperty) {
      
      if !(showingProperties[property] ?? false) {
        continue
      }
      
      var dataEntries = [ChartDataEntry]()
      for (index, value) in trainingAnalysis.analyseProperty(property, inTimeRange: timeRange).enumerate() {
        dataEntries.append(ChartDataEntry(value: value, xIndex: index))
      }
      
      
      let dataSet = LineChartDataSet(yVals: dataEntries, label: nil)
      let propertyColor = property.propertyColor
      
      
      // Line draw
      
      dataSet.colors = [propertyColor.colorWithAlphaComponent(0.8)]
//      dataSet.drawCubicEnabled = true
      
      
      // Space under the line
      
      dataSet.drawFilledEnabled = true
      dataSet.fillAlpha = 0.8
      dataSet.fillColor = propertyColor
      
      
      // Value circles
      
      dataSet.circleRadius = 5.0
      dataSet.valueTextColor = UIColor.blackColor()
      dataSet.setCircleColor(propertyColor)
      dataSet.drawCircleHoleEnabled = false
      dataSet.drawValuesEnabled = true
      
      dataSets.append(dataSet)
    }
    
    return LineChartData(xVals: xVals, dataSets: dataSets)
  }
  
}
