

import UIKit

@IBDesignable final class SegmentedControl: UIControl {
  
  private var labels = [UILabel]()
  private var line = CALayer()
  
  var thumbView = UIView()
  
  
  var items: [String] = ["7", "14", "30", "90"] {
    didSet {
      setupLabels()
    }
  }
  
  var selectedIndex : Int = 0 {
    didSet {
      displayNewSelectedIndex()
    }
  }
  
  @IBInspectable var selectedLabelColor : UIColor = UIColor.blackColor() {
    didSet {
      setSelectedColors()
    }
  }
  
  @IBInspectable var unselectedLabelColor : UIColor = UIColor.whiteColor() {
    didSet {
      setSelectedColors()
    }
  }
  
  @IBInspectable var thumbColor : UIColor = UIColor.whiteColor() {
    didSet {
      setSelectedColors()
    }
  }
  
  @IBInspectable var borderColor : UIColor = UIColor.whiteColor() {
    didSet {
      layer.borderColor = borderColor.CGColor
    }
  }
  
  @IBInspectable var font : UIFont! = UIFont.systemFontOfSize(18) {
    didSet {
      setFont()
    }
  }
  
  override init(frame: CGRect) {
    super.init(frame: frame)
    
    setupView()
  }
  
  required init?(coder: NSCoder) {
    super.init(coder: coder)
    setupView()
  }
  
  func setupView(){
    
    backgroundColor = UIColor.clearColor()
    
    self.layer.addSublayer(line)
    
    setupLabels()
    
    insertSubview(thumbView, atIndex: 0)
    
  }
  
  func setupLabels(){
    
    for label in labels {
      label.removeFromSuperview()
    }
    
    labels.removeAll(keepCapacity: true)
    
    for index in 1...items.count {
      
      let label = UILabel(frame: CGRectMake(0, 0, 30, 40))
      label.text = items[index - 1]
      label.backgroundColor = UIColor.whiteColor()
      label.textAlignment = .Center
      label.font = UIFont(name: "Avenir-Black", size: 15)
      label.textColor = index == 1 ? selectedLabelColor : unselectedLabelColor
      self.addSubview(label)
      labels.append(label)
    }
    
  }
  
  override func layoutSubviews() {
    super.layoutSubviews()
    
    var selectFrame = self.bounds
    selectFrame.size.width = selectFrame.height
    thumbView.frame = selectFrame
    thumbView.backgroundColor = thumbColor
    thumbView.layer.cornerRadius = thumbView.frame.height / 2
    
    for (index, label) in labels.enumerate() {
      label.center = CGPoint(x: CGFloat(index) * (self.bounds.width - label.frame.width) / CGFloat(labels.count-1) + label.frame.width/2, y: self.bounds.midY)
    }
    
    displayNewSelectedIndex()
    
    var lineFrame = self.bounds
    lineFrame.size.height = 1
    lineFrame.origin.y = self.bounds.midY - lineFrame.height/2
    line.frame = lineFrame
    
    line.backgroundColor = unselectedLabelColor.CGColor
  }
  
  override func beginTrackingWithTouch(touch: UITouch, withEvent event: UIEvent?) -> Bool {
    
    let location = touch.locationInView(self)
    
    var calculatedIndex : Int?
    for (index, item) in labels.enumerate() {
      if item.frame.contains(location) {
        calculatedIndex = index
      }
    }
    
    
    if calculatedIndex != nil {
      selectedIndex = calculatedIndex!
      sendActionsForControlEvents(.ValueChanged)
    }
    
    return false
  }
  
  func displayNewSelectedIndex(){
    for item in labels {
      item.textColor = unselectedLabelColor
      item.backgroundColor = UIColor.whiteColor()
    }
    
    let label = labels[selectedIndex]
    label.textColor = selectedLabelColor
    label.backgroundColor = UIColor.clearColor()
    
    self.thumbView.center = label.center
    
  }
  
  func setSelectedColors(){
    for item in labels {
      item.textColor = unselectedLabelColor
    }
    
    if labels.count > 0 {
      labels[0].textColor = selectedLabelColor
    }
    
    thumbView.backgroundColor = thumbColor
  }
  
  func setFont(){
    for item in labels {
      item.font = font
    }
  }
}
