//
//  UserEntityTest.swift
//  Trampigo
//
//  Created by Naim Ashhab on 13.01.16.
//  Copyright © 2016 Naim Ashhab. All rights reserved.
//

import XCTest
import SwiftyJSON

class UserEntityTest: XCTestCase {
  
  // MARK: - User
  
  func testInit() {
    
    let json = JSON([
      "fullName" : "test name",
      "email" : "test email",
      "id" : "test id"
    ])
    guard let user = User(json: json, accessToken: "test access token") else {
      return XCTAssert(false)
    }
    
    XCTAssert(user.id == "test id")
    XCTAssert(user.email == "test email")
    XCTAssert(user.fullName == "test name")
    XCTAssert(user.accessToken == "test access token")
  }
  
  func testInvalidJSONInit() {
    var json = JSON([
      //      "fullName" : "test name",
      "email" : "test email",
      "id" : "test id"
    ])
    
    if let _ = User(json: json, accessToken: nil) {
      return XCTAssert(false)
    }
    
    
    json = JSON([
      "fullName" : "test name",
      //      "email" : "test email",
      "id" : "test id"
    ])
    
    if let _ = User(json: json, accessToken: nil) {
      return XCTAssert(false)
    }
    
    
    json = JSON([
      "fullName" : "test name",
      "email" : "test email",
      //      "id" : "test id"
    ])
    
    if let _ = User(json: json, accessToken: nil) {
      return XCTAssert(false)
    }
  }
  
  
  // MARK: - Coach
  
  func testCoachInit() {
    
    let json = JSON([
      "fullName" : "test name",
      "email" : "test email",
      "id" : "test id",
      "connections" : [
        "gymnasts" : [
          ["id" : "1", "email": "g@1", "fullName": "G1"],
          ["id" : "2", "email": "g@2", "fullName": "G2"],
          ["id" : "3", "email": "g@3", "fullName": "G3"]
        ]
      ]
      ])
    guard let user = Coach(json: json, accessToken: "test access token") else {
      return XCTAssert(false)
    }
    
    XCTAssert(user.id == "test id")
    XCTAssert(user.email == "test email")
    XCTAssert(user.fullName == "test name")
    XCTAssert(user.accessToken == "test access token")
    
    XCTAssert(user.gymnasts.count == 3)
    XCTAssert(user.gymnasts[1].fullName == "G2")
    XCTAssert(user.gymnasts[0].email == "g@1")
  }
  
  func testCoachNoGymnastsInit() {
    let json = JSON([
      "fullName" : "test name",
      "email" : "test email",
      "id" : "test id"
      ])
    guard let user = Coach(json: json, accessToken: "test access token") else {
      return XCTAssert(false)
    }
    
    XCTAssert(user.gymnasts.count == 0)
  }
  
  // MARK: - Gymnast
  
  func testGymnastInit() {
    
    let json = JSON([
      "fullName" : "test name",
      "email" : "test email",
      "id" : "test id",
      "connections" : [
        "coach" : ["id" : "1", "email": "c@1", "fullName": "C1"]
      ]
    ])
    guard let user = Gymnast(json: json, accessToken: "test access token") else {
      return XCTAssert(false)
    }
    
    XCTAssert(user.id == "test id")
    XCTAssert(user.email == "test email")
    XCTAssert(user.fullName == "test name")
    XCTAssert(user.accessToken == "test access token")
    
    XCTAssert(user.coach != nil)
    XCTAssert(user.coach!.fullName == "C1")
    XCTAssert(user.coach!.email == "c@1")
  }
  
  func testGymnastNoCoachInit() {
    
    let json = JSON([
      "fullName" : "test name",
      "email" : "test email",
      "id" : "test id"
    ])
    
    XCTAssert(json["connections"]["coach"] == JSON.null)
    
    guard let user = Gymnast(json: json, accessToken: "test access token") else {
      return XCTAssert(false)
    }
    
    XCTAssert(user.coach == nil)
  }
  
  func testUserEquality() {
    
    let json = JSON([
      "fullName" : "test name",
      "email" : "test email",
      "id" : "test id"
    ])
    
    let user1 = User(json: json, accessToken: nil)!
    let user2 = User(json: json, accessToken: nil)!
    
    XCTAssert(user1.isEqualTo(user2))
    
    user1.fullName = "test name 2"
    XCTAssert(!user1.isEqualTo(user2))
  }
  
  func testGymnastEquality() {
    let json = JSON([
      "fullName" : "test name",
      "email" : "test email",
      "id" : "test id",
      "connections" : [
        "coach" : ["id":"", "email": "", "fullName": ""]
      ]
    ])
    
    let user1 = Gymnast(json: json, accessToken: nil)!
    let user2 = Gymnast(json: json, accessToken: nil)!
    
    XCTAssert(user1.isEqualTo(user2))
    
    user1.coach = nil
    XCTAssert(!user1.isEqualTo(user2))
    
    user1.coach = Coach(json: JSON(["id":"", "email": "", "fullName": ""]), accessToken: nil)
    XCTAssert(user1.isEqualTo(user2))
    
    user1.coach?.fullName = "test"
    XCTAssert(!user1.isEqualTo(user2))
  }
  
  func testCoachEquality() {
    let json = JSON([
      "fullName" : "test name",
      "email" : "test email",
      "id" : "test id",
      "connections" : [
        "gymnasts" : [
          ["id":"", "email": "", "fullName": ""],
          ["id":"", "email": "", "fullName": ""]
        ]
      ]
    ])
    
    let user1 = Coach(json: json, accessToken: nil)!
    let user2 = Coach(json: json, accessToken: nil)!
    
    XCTAssert(user1.isEqualTo(user2))
    
    user1.gymnasts.removeLast()
    XCTAssert(!user1.isEqualTo(user2))
    
    user1.gymnasts.append(Gymnast(json: JSON(["id":"", "email": "", "fullName": ""]), accessToken: nil)!)
    XCTAssert(user1.isEqualTo(user2))
    
    user1.gymnasts[0].fullName = "test name 2"
    XCTAssert(!user1.isEqualTo(user2))
    
  }
  
}
